PageFlags

Struct PageFlags 

pub struct PageFlags { /* private fields */ }
Page protection permissions and access status.

Implementations

Source

impl PageFlags

pub open spec fn flags_spec(&self) -> Set<__ghostPageFlags>

{ Self::flags_from_bits(self.bits()) }
pub open spec fn flags_from_bits(bits: u8) -> Set<__ghostPageFlags>

{
    ::vstd::set::Set::<
        __ghostPageFlags,
    >::from_finite_type(|flag: __ghostPageFlags| {
        flag.enabled() && (bits & flag.bit()) == flag.bit()
    })
}
pub closed spec fn R_spec() -> Self

pub const exec fn R() -> r : Self

ensures
r.bits() == (0b00000001),
r.flags_spec() == Self::flags_from_bits((0b00000001) as u8),
returns
Self::R_spec(),

Readable.

pub closed spec fn W_spec() -> Self

pub const exec fn W() -> r : Self

ensures
r.bits() == (0b00000010),
r.flags_spec() == Self::flags_from_bits((0b00000010) as u8),
returns
Self::W_spec(),

Writable.

pub closed spec fn X_spec() -> Self

pub const exec fn X() -> r : Self

ensures
r.bits() == (0b00000100),
r.flags_spec() == Self::flags_from_bits((0b00000100) as u8),
returns
Self::X_spec(),

Executable.

pub closed spec fn RW_spec() -> Self

pub const exec fn RW() -> r : Self

ensures
r.bits() == (0b00000011),
r.flags_spec() == Self::flags_from_bits((0b00000011) as u8),
returns
Self::RW_spec(),

Readable + writable.

pub closed spec fn RX_spec() -> Self

pub const exec fn RX() -> r : Self

ensures
r.bits() == (0b00000101),
r.flags_spec() == Self::flags_from_bits((0b00000101) as u8),
returns
Self::RX_spec(),

Readable + executable.

pub closed spec fn RWX_spec() -> Self

pub const exec fn RWX() -> r : Self

ensures
r.bits() == (0b00000111),
r.flags_spec() == Self::flags_from_bits((0b00000111) as u8),
returns
Self::RWX_spec(),

Readable + writable + executable.

pub closed spec fn ACCESSED_spec() -> Self

pub const exec fn ACCESSED() -> r : Self

ensures
r.bits() == (0b00001000),
r.flags_spec() == Self::flags_from_bits((0b00001000) as u8),
returns
Self::ACCESSED_spec(),

Has the memory page been read or written.

pub closed spec fn DIRTY_spec() -> Self

pub const exec fn DIRTY() -> r : Self

ensures
r.bits() == (0b00010000),
r.flags_spec() == Self::flags_from_bits((0b00010000) as u8),
returns
Self::DIRTY_spec(),

Has the memory page been written.

pub closed spec fn AVAIL1_spec() -> Self

pub const exec fn AVAIL1() -> r : Self

ensures
r.bits() == (0b01000000),
r.flags_spec() == Self::flags_from_bits((0b01000000) as u8),
returns
Self::AVAIL1_spec(),

The first bit available for software use.

pub closed spec fn AVAIL2_spec() -> Self

pub const exec fn AVAIL2() -> r : Self

ensures
r.bits() == (0b10000000),
r.flags_spec() == Self::flags_from_bits((0b10000000) as u8),
returns
Self::AVAIL2_spec(),

The second bit available for software use.

pub proof fn lemma_all_constant()

ensures
Self::all().bits() == Self::all_spec().bits_spec(),
Self::all_spec().bits_spec()
    == (0 as u8)
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable."]) true
        } {
            ((0b00000001) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Writable."]) true
        } {
            ((0b00000010) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Executable."]) true
        } {
            ((0b00000100) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable."]) true
        } {
            ((0b00000011) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + executable."]) true
        } {
            ((0b00000101) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable + executable."]) true
        } {
            ((0b00000111) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been read or written."]) true
        } {
            ((0b00001000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been written."]) true
        } {
            ((0b00010000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The first bit available for software use."]) true
        } {
            ((0b01000000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The second bit available for software use."]) true
        } {
            ((0b10000000) as u8)
        } else {
            0 as u8
        }),
Self::all().bits()
    == (0 as u8)
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable."]) true
        } {
            ((0b00000001) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Writable."]) true
        } {
            ((0b00000010) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Executable."]) true
        } {
            ((0b00000100) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable."]) true
        } {
            ((0b00000011) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + executable."]) true
        } {
            ((0b00000101) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable + executable."]) true
        } {
            ((0b00000111) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been read or written."]) true
        } {
            ((0b00001000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been written."]) true
        } {
            ((0b00010000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The first bit available for software use."]) true
        } {
            ((0b01000000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The second bit available for software use."]) true
        } {
            ((0b10000000) as u8)
        } else {
            0 as u8
        }),
pub broadcast proof fn lemma_consts()

ensures
Self::all().bits()
    == (0 as u8)
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable."]) true
        } {
            ((0b00000001) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Writable."]) true
        } {
            ((0b00000010) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Executable."]) true
        } {
            ((0b00000100) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable."]) true
        } {
            ((0b00000011) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + executable."]) true
        } {
            ((0b00000101) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Readable + writable + executable."]) true
        } {
            ((0b00000111) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been read or written."]) true
        } {
            ((0b00001000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" Has the memory page been written."]) true
        } {
            ((0b00010000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The first bit available for software use."]) true
        } {
            ((0b01000000) as u8)
        } else {
            0 as u8
        })
        | (if $crate::__bitflags_cfg_expr! {
            (#[doc = r" The second bit available for software use."]) true
        } {
            ((0b10000000) as u8)
        } else {
            0 as u8
        }),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Readable."]) Self::R().bits() == ((0b00000001) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Writable."]) Self::W().bits() == ((0b00000010) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Executable."]) Self::X().bits() == ((0b00000100) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Readable + writable."]) Self::RW().bits() == ((0b00000011) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Readable + executable."]) Self::RX().bits() == ((0b00000101) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Readable + writable + executable."]) Self::RWX().bits() == ((0b00000111)
    as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Has the memory page been read or written."]) Self::ACCESSED().bits() ==
    ((0b00001000) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" Has the memory page been written."]) Self::DIRTY().bits() ==
    ((0b00010000) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" The first bit available for software use."]) Self::AVAIL1().bits() ==
    ((0b01000000) as u8)
),
$crate::__bitflags_cfg_guarded_expr!(
    (#[doc = r" The second bit available for software use."]) Self::AVAIL2().bits() ==
    ((0b10000000) as u8)
),
pub proof fn lemma_from_bits_bits(bits: u8)

requires
bits & Self::all().bits() == bits,
ensures
Self::from_bits(bits)->0.bits() == bits,
pub proof fn lemma_eq_from_bits(left: Self, right: Self)

requires
left.bits() == right.bits(),
ensures
left == right,
pub closed spec fn bits_spec(&self) -> u8

The raw bits stored inside this flags value.

pub const exec fn bits(&self) -> u8

returns
self.bits(),
pub closed spec fn empty_spec() -> Self

pub const exec fn empty() -> r : Self

ensures
r.bits() == 0,
r.flags_spec() == Self::flags_from_bits(0),
returns
Self::empty(),
pub closed spec fn all_spec() -> Self

pub const exec fn all() -> r : Self

ensures
r == Self::all_spec(),
r.bits() == Self::all().bits(),
r.flags_spec() == Self::flags_from_bits(Self::all().bits()),
pub closed spec fn known_bits_spec(&self) -> u8

The bits in self that correspond to declared flags.

pub const exec fn known_bits(&self) -> r : u8

returns
self.known_bits(),
pub closed spec fn unknown_bits_spec(&self) -> u8

The bits in self that do not correspond to declared flags.

pub const exec fn unknown_bits(&self) -> r : u8

returns
self.unknown_bits(),
pub open spec fn contains_unknown_bits_spec(&self) -> bool

{ self.unknown_bits() != 0 }

This method returns true if any unknown bits are set.

pub const exec fn contains_unknown_bits(&self) -> r : bool

returns
self.contains_unknown_bits(),
pub open spec fn is_empty_spec(&self) -> bool

{ self.bits() == 0 }

Whether all bits in self are unset.

pub const exec fn is_empty(&self) -> r : bool

returns
self.is_empty(),
pub open spec fn is_all_spec(&self) -> bool

{ Self::all().bits() | self.bits() == self.bits() }

Whether all known bits are set.

pub const exec fn is_all(&self) -> r : bool

returns
self.is_all(),
pub open spec fn contains_spec(&self, other: Self) -> bool

{ (self.bits() & other.bits()) == other.bits() }

Whether all set bits in other are also set in self.

pub open spec fn contains_flags_spec(&self, other: Self) -> bool

{ self.contains(other) }
pub const exec fn contains(&self, other: Self) -> r : bool

returns
self.contains(other),
pub open spec fn intersects_spec(&self, other: Self) -> bool

{ (self.bits() & other.bits()) != 0 }

Whether any set bits in other are also set in self.

pub const exec fn intersects(&self, other: Self) -> r : bool

returns
self.intersects(other),
pub closed spec fn from_bits_truncate_spec(bits: u8) -> Self

pub closed spec fn from_bits_retain_spec(bits: u8) -> Self

pub const exec fn from_bits_retain(bits: u8) -> r : Self

ensures
r.bits() == bits,
r.flags_spec() == Self::flags_from_bits(bits),
returns
Self::from_bits_retain(bits),
pub exec fn from_name(name: &str) -> r : Option<Self>

ensures
r matches Some(
    flags_value,
) ==> {
    &&& flags_value.flags_spec() == Self::flags_from_bits(flags_value.bits())

},

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const exec fn from_bits_truncate(bits: u8) -> r : Self

ensures
r.bits() == (bits & Self::all().bits()),
r.flags_spec() == Self::flags_from_bits(bits & Self::all().bits()),
returns
Self::from_bits_truncate(bits),
pub closed spec fn from_bits_spec(bits: u8) -> Option<Self>

pub const exec fn from_bits(bits: u8) -> r : Option<Self>

ensures
r is Some == ((bits & Self::all().bits()) == bits),
r matches Some(
    flags_value,
) ==> {
    &&& flags_value.bits() == bits
    &&& flags_value.flags_spec() == Self::flags_from_bits(bits)

},
returns
Self::from_bits(bits),
pub closed spec fn remove_spec(self, other: Self) -> Self

pub exec fn insert(&mut self, other: Self)

ensures
final(self).bits() == (old(self).bits() | other.bits()),
final(self).flags_spec() == Self::flags_from_bits(old(self).bits() | other.bits()),
pub exec fn remove(&mut self, other: Self)

ensures
*final(self) == old(self).remove_spec(other),
final(self).bits() == (old(self).bits() & !other.bits()),
final(self).flags_spec() == Self::flags_from_bits(old(self).bits() & !other.bits()),
pub exec fn toggle(&mut self, other: Self)

ensures
final(self).bits() == (old(self).bits() ^ other.bits()),
final(self).flags_spec() == Self::flags_from_bits(old(self).bits() ^ other.bits()),

The bitwise exclusive-or (^) of the bits in self and other.

pub exec fn set(&mut self, other: Self, value: bool)

ensures
value ==> final(self).bits() == (old(self).bits() | other.bits()),
!value ==> final(self).bits() == (old(self).bits() & !other.bits()),

Call insert when value is true or remove when value is false.

pub exec fn truncate(&mut self)

ensures
final(self).bits() == (old(self).bits() & Self::all().bits()),
final(self).flags_spec() == Self::flags_from_bits(old(self).bits() & Self::all().bits()),

Remove any unknown bits from the flags.

pub exec fn clear(&mut self)

ensures
final(self).bits() == 0,
final(self).flags_spec() == Self::flags_from_bits(0),

Unsets all bits in the flags.

pub closed spec fn union_spec(self, other: Self) -> Self

pub const exec fn union(self, other: Self) -> r : Self

ensures
r.bits() == (self.bits() | other.bits()),
returns
self.union(other),
pub closed spec fn intersection_spec(self, other: Self) -> Self

pub const exec fn intersection(self, other: Self) -> r : Self

ensures
r.bits() == (self.bits() & other.bits()),
returns
self.intersection(other),
pub closed spec fn difference_spec(self, other: Self) -> Self

pub const exec fn difference(self, other: Self) -> r : Self

ensures
r.bits() == (self.bits() & !other.bits()),
returns
self.difference(other),
pub closed spec fn symmetric_difference_spec(self, other: Self) -> Self

pub const exec fn symmetric_difference(self, other: Self) -> r : Self

ensures
r.bits() == (self.bits() ^ other.bits()),
returns
self.symmetric_difference(other),
pub closed spec fn complement_spec(self) -> Self

pub const exec fn complement(self) -> r : Self

ensures
r.bits() == (!self.bits() & Self::all().bits()),
r.flags_spec() == Self::flags_from_bits(!self.bits() & Self::all().bits()),
returns
self.complement(),