HasDaddr

Trait HasDaddr 

pub trait HasDaddr {
    // Required method
    exec fn daddr(&self) -> Daddr;
}
A trait for types that have mapped address in the device address space.

Required Methods

exec fn daddr(&self) -> Daddr

Gets the base address of the mapping in the device address space.

Implementors