pub struct PageProperty {
pub flags: PageFlags,
pub cache: CachePolicy,
pub priv_flags: PrivilegedPageFlags,
}Expand description
The property of a mapped virtual memory page.
Fields
flags: PageFlagsThe flags associated with the page,
cache: CachePolicyThe cache policy for the page.
priv_flags: PrivilegedPageFlagsImplementations
Sourceimpl PageProperty
impl PageProperty
Sourcepub exec fn new_user(flags: PageFlags, cache: CachePolicy) -> Self
pub exec fn new_user(flags: PageFlags, cache: CachePolicy) -> Self
returns
Self::new_user(flags, cache),Creates a new PageProperty with the given flags and cache policy for the user.
pub fn __VERUS_SPEC_new_user(flags: PageFlags, cache: CachePolicy) -> Self
Sourcepub exec fn new_absent() -> Self
pub exec fn new_absent() -> Self
returns
Self::new_absent(),Creates a page property that implies an invalid page without mappings.
pub fn __VERUS_SPEC_new_absent() -> Self
Sourceimpl PageProperty
impl PageProperty
Sourcepub proof fn lemma_avail1_tag_encoding(self)
pub proof fn lemma_avail1_tag_encoding(self)
ensures
self.flags.union(PageFlags::AVAIL1()).contains(PageFlags::AVAIL1()),!self.flags.difference(PageFlags::AVAIL1()).contains(PageFlags::AVAIL1()),!self.flags.contains(PageFlags::AVAIL1())
==> self.flags.difference(PageFlags::AVAIL1()) == self.flags,!self.flags.contains(PageFlags::AVAIL1())
==> self.flags.union(PageFlags::AVAIL1()).difference(PageFlags::AVAIL1())
== self.flags,self.flags.contains(PageFlags::AVAIL1())
==> self.flags.difference(PageFlags::AVAIL1()).union(PageFlags::AVAIL1())
== self.flags,Adding and removing AVAIL1 is reversible when it is used as a reserved tag.